from isharp.datahub.core import StorageMethod,MatrixHeader,Revision,AcquireContentReturnValue,MemStyles,RevisionInfo
from typing import List
import logging
import dataclasses
logging.basicConfig(level=logging.INFO)
class ArcticStorageMethod(StorageMethod):

    def __init__(self,store):
        self.store = store
        super().__init__("arctic")

    def _lib_ticker(self,path):
        tokens  = [ i for i in list(path.split('/')) if len(i)>0]
        library = tokens[0]
        ticker = '.'.join(tokens[1:])
        logging.info("Arctic storage method converted path to libaray [{}], ticker [{}]".format(library,ticker))

        if  library in self.store.list_libraries():
            lib = self.store[library]
            if lib.has_symbol(ticker):
                versioned  = lib.read(ticker)
                return (library,ticker)

            else:
                raise StorageMethod.ResourceException("ticker {} not found".format(ticker))
        else:
            raise StorageMethod.ResourceException("library {} not found in library list {}".format(library,self.store.list_libraries()))

        return (library, ticker)

    def acquireContent(self, path, params, version_id=None)->AcquireContentReturnValue:
        library, ticker = self._lib_ticker(path)
        lib = self.store[library]
        v = None if version_id is None else int(version_id)
        versioned  = lib.read(ticker, v)
        header = MatrixHeader(
        name= path,
        revision_id= str(versioned.version),
        storage_method=self.name,
        path= path,
        memory_style=MemStyles.DATA_FRAME,
        description=None
        )


        return AcquireContentReturnValue(content=versioned.data,header=header)




    def storeContent(self, path, params, content,revision_info)->Revision:
        library, ticker = self._lib_ticker(path)
        _store_content(self.store[library],ticker,content,revision_info)

    def history(self,matrix_url)->List[Revision]:
        library, ticker = self._lib_ticker(matrix_url.url_components.path)
        lib = self.store[library]
        meta = lib.read_metadata(ticker)
        logging.info("attempted to get history for : {},{} result = [{}]".format(library, ticker,meta))
        if meta.metadata is None:
            return []
        else:
            return get_revisions_from_metadata(meta.metadata)


    def list(self) -> List[MatrixHeader]:
        ret_val = []
        for this_lib_name in self.store.list_libraries():
            library = self.store[this_lib_name]
            for this_symbol in library.list_symbols():
                versions = library.list_versions(this_symbol)
                filtered = [version for version in versions if not version['deleted']]
                max_version = max(map(lambda v: v['version'], filtered))
                symbol_with_slashes = this_symbol.replace('.','/')
                ret_val.append(MatrixHeader(name=symbol_with_slashes,
                                            description="don't know yet",
                                            storage_method = self.name,
                                            memory_style = MemStyles.DATA_FRAME,
                                            revision_id = str(max_version),
                                            path="{}/{}".format(this_lib_name,symbol_with_slashes)))

        return ret_val




history_tag = "revision_history"

def add_revision_to_metadata(revision:Revision,metadata:dict,dict_key:str=history_tag):
    if metadata.get(dict_key) is None:
        metadata[dict_key] = []
    metadata[dict_key].append(dataclasses.asdict(revision))

def _revision_from_dict(dict:dict)->Revision:
    #todo ... there must be a better whay of doing this !
    revision_id = dict['id']
    revision_info = dict["revision_info"]
    return Revision(revision_id,RevisionInfo(who=revision_info['who'],what=revision_info['what'],when=revision_info['when']))


def get_revisions_from_metadata(metadata:dict,dict_key:str=history_tag)->List[Revision]:
    revision_list = metadata[dict_key]
    logging.info("retrieved revision list from  metadata: {}".format(revision_list))
    if revision_list is not  None:
        return [_revision_from_dict(i) for i in revision_list]
    else:
        return []



def import_pandas(lib, pd, symbol_name,revision_info):
    meta = {}
    add_revision_to_metadata(Revision('1', revision_info), meta)
    lib.write(symbol_name, pd, meta)


def _store_content(lib,ticker,content,revision_info)->Revision:
        original_meta_d = lib.read_metadata(ticker).metadata

        old_revisions = get_revisions_from_metadata(original_meta_d)
        last_revision = old_revisions[-1]
        next_revision_id = str(int(last_revision.id)+1)
        new_revision = Revision(next_revision_id,revision_info)
        add_revision_to_metadata(new_revision,original_meta_d)
        ret = lib.write(ticker,content,original_meta_d)
        return new_revision




from typing import List
from nameko.standalone.rpc import ClusterRpcProxy
import logging
import time
from isharp.evalengine.core import Evaluator,EvalMethod
from isharp.datahub.core import  DatahubTarget
logger = logging.getLogger(__name__)

def remote_config(net_location: str):
    return {
        'serializer': 'pickle',
        'AMQP_URI': 'pyamqp://guest:guest@{}'.format(net_location),
        'rpc_exchange': 'nameko-rpc',
        'max_workers': 10,
        'parent_calls_tracked': 10
    }

class AsyncEvalServiceInvoker:
    def __init__(self, conf):
        self.rpc_proxy = ClusterRpcProxy(conf)
        self.proxy = self.rpc_proxy.start()

    def eval(self, method: EvalMethod, inputs: List[DatahubTarget]) -> List[DatahubTarget]:
        result = self.proxy.evaluation_service.eval(method, inputs)
        print (result)




    def stop(self):
        self.rpc_proxy.stop()





invoker = AsyncEvalServiceInvoker(remote_config("localhost"))

invoker.eval(None,[])


from nameko.extensions import DependencyProvider
from nameko.rpc import rpc, RpcProxy
from isharp.datahub.core import  MatrixHeader,CombiBroker
from typing import List
from isharp.datahub.core import  DataBroker,Matrix,RevisionInfo,Revision
import logging


handler = logging.StreamHandler()
handler.setLevel(logging.INFO)
formatter = logging.Formatter("%(levelname)s - %(message)s")
handler.setFormatter(formatter)
logger = logging.getLogger(__name__)
logger.addHandler(handler)

class DataBrokerDelegate(DependencyProvider):

    def get_dependency(self, worker_ctx):
        print("getting data broker delegates")
        data_brokers =worker_ctx.container.config.get('data_brokers')
        print("got {} data brokers".format(len(data_brokers)))
        return  CombiBroker(data_brokers)

class DataBrokerService(DataBroker):
    name="data_broker_service"
    data_broker_rpc = RpcProxy("data_broker")
    delegate = DataBrokerDelegate()

    @rpc
    def checkout(self, url: str, version_id=None) -> Matrix:
        logger.debug("delegate received checkout call")
        ret_val = self.delegate.checkout(url,version_id)
        logger.debug("checkout call to delegate successful")
        return ret_val

    @rpc
    def view(self, url: str, version_id=None) -> Matrix:
        logger.debug("delegate received view call")
        ret_val = self.delegate.view(url,version_id)
        logger.debug("view call to delegate successful")
        return ret_val

    @rpc
    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        logger.debug("delegate received commit call")
        return super().commit(matrix, revisionInfo)
    @rpc
    def release(self, matrix) -> None:
        logger.debug("delegate received release call")
        self.delegate.release(matrix)
        logger.debug("release call to delegate successful")
        return None

    @rpc
    def releaseAll(self) -> None:
        logger.debug("delegate reeived release all call")
        self.delegate.releaseAll()
        logger.debug("release call to delegate successful")
        return None

    @rpc
    def list(self) -> List[MatrixHeader]:
        logger.debug("delegate about to retrieve listing")
        return self.delegate.list()

    @rpc
    def peek(self, url) -> MatrixHeader:
        logger.debug("delegate about to peek")
        return self.delegate.peek(url)

    @rpc
    def history(self,url: str) -> List[Revision]:
        logger.debug("delegate history call")
        return self.delegate.history(url)












import os
import test.testutil.pandas_utils as pu
import pandas as pd
import pathlib
rows = ['a','b','c','d','e','f','g','h','i','j','k']

def fill_directory_with_content(root_directory,directory_path, number_files):
    index_data =[]
    key_data = []
    for i in range(1, number_files):
        df = pu.create_simple_series(rows[:i + 2], i + 10)
        file_name = "file_name_{}.csv".format(i)
        filepath = os.path.join(root_directory,directory_path, file_name)
        df.to_csv(filepath)
        posix_path  = pathlib.PureWindowsPath(os.path.join("/",directory_path,file_name))
        key_data.append(posix_path.as_posix().strip("/"))
        index_data.append((file_name,"description of {}".format(file_name)))

    df = pd.DataFrame( index=pd.Index(key_data,name="path"),data=index_data, columns=["name", "description"])

    index_file_path = os.path.join(root_directory,directory_path, "index.txt")
    df.to_csv(index_file_path)


def make_file_tree(root_directory_path,tree_depth, files_per_directory,relative_path=''):

    for i in range(1, files_per_directory):
        fill_directory_with_content(root_directory_path,relative_path,files_per_directory)
        if tree_depth > 0:
            new_relative_path = os.path.join(relative_path,"subdir_{}".format(i))
            actual_directory_path =os.path.join(root_directory_path,new_relative_path)
            os.mkdir(actual_directory_path)
            make_file_tree(root_directory_path, tree_depth-1, files_per_directory,new_relative_path)










from py2neo import  Graph
from isharp.flow.core import CalculationTask, DatahubRequirement




url = "bolt://ec2-34-205-159-121.compute-1.amazonaws.com:7687"


def calcTasks(data_hub_host_name, data_hub_port):
    ret_val = []
    graph = Graph(url)
    strategies = graph.nodes.match('Strategy')
    for strategy in strategies:
        jobs = graph.match((strategy,), r_type='EVAL_JOB')
        for j in jobs:
            deps = []
            job_node = j.end_node
            dependencies = graph.match((job_node,), r_type="USES")
            for dependency in dependencies:
                datahub_url = "{}://{}:{}/{}".format("arctic",data_hub_host_name,data_hub_port,dependency.end_node['path'])
                deps.append(DatahubRequirement(name=dependency.end_node['name'],t=0,url=datahub_url) )
            ret_val.append(CalculationTask(dueBy=job_node['due_by'],requirements=deps,strategy=strategy['name'],eval_label=job_node['name']))

    return ret_val






import unittest
import test.testutil.mongo_arctic_utils as marc
import test.testutil.log_utils as lu
import test.testutil.pandas_utils as pu
import random
import string
from arctic import Arctic
import pandas as pd
import numpy as np
import datetime
from isharp.datahub.arctic_broker.broker_impl.arctic_storage_method import  ArcticStorageMethod, add_revision_to_metadata,get_revisions_from_metadata,import_pandas
from isharp.datahub.core import StorageMethod, MatrixUrl,Revision,RevisionInfo
from test.testutil import randomString




class TestMongoBroker(unittest.TestCase):
    def setUp(self):
        self.username = randomString()
        self.time_now = datetime.datetime.now()
        self.arctic = Arctic('localhost')
        self.library_name = marc.random_library(self.arctic)
        self.symbol_name = 'symbol'
        self.qualified_symbol_name = "ES.SETL.EOD"
        self.qualified_symbol_name_as_path = self.qualified_symbol_name.replace('.','/')
        lu.logger.info("Created test library {}".format(self.library_name))
        self.arctic.initialize_library(self.library_name)
        pd = pu.create_simple_series(['a','b','c'] , 50)
        import_pandas(self.arctic[self.library_name],pd,self.symbol_name,RevisionInfo(who=self.username, what="init", when=self.time_now))
        import_pandas(self.arctic[self.library_name],pu.create_simple_series(['a','b','c','d'] , 75),self.qualified_symbol_name,
                      RevisionInfo(who=self.username, what="init qualified", when=self.time_now))

    def tearDown(self):
        self.arctic.delete_library(self.library_name)
        lu.logger.info("deleted test library {}".format(self.library_name))

    def test_acquire_data(self):
        method = ArcticStorageMethod(self.arctic)
        result  = method.acquireContent("{}/symbol".format(self.library_name),{})
        data_frame = result.content
        self.assertFalse(result.header.path[0] == '/')
        self.assertIs(3,len(data_frame.columns))
        self.assertIsNone(result.header.description)
        self.assertEqual(result.header.name,result.header.path)


    def test_history(self):
        method = ArcticStorageMethod(self.arctic)
        url = MatrixUrl("{}/symbol".format(self.library_name))
        result = method.history(url)
        self.assertEquals(1,len(result))


    def test_acquire_data_with_qualified_symbol(self):
        method = ArcticStorageMethod(self.arctic)
        url = "{}/{}".format(self.library_name,self.qualified_symbol_name_as_path)
        result  = method.acquireContent(url,{})
        data_frame = result.content
        self.assertIs(4,len(data_frame.columns))


    def test_revision_storage(self):
        time_now = datetime.datetime.now()
        info = RevisionInfo(who="aPerson",what="a Change", when=time_now)

        metadata = {}
        add_revision_to_metadata(Revision("1",info),metadata)
        info2 = RevisionInfo(who="anotherPerson", what="anotherChange", when=time_now)
        add_revision_to_metadata( Revision("2", info2), metadata)

        revs = get_revisions_from_metadata(metadata)
        self.assertEquals(2,len(revs))
        revision = revs[0]
        self.assertEquals("1",revision.id)
        self.assertEquals("a Change", revision.revision_info.what)
        self.assertEquals("aPerson", revision.revision_info.who)
        self.assertEquals(time_now, revision.revision_info.when)

        revision = revs[1]
        self.assertEquals("2",revision.id)
        self.assertEquals("anotherChange", revision.revision_info.what)
        self.assertEquals("anotherPerson", revision.revision_info.who)
        self.assertEquals(time_now, revision.revision_info.when)



    def test_acquire_missing_data(self):
        method = ArcticStorageMethod(self.arctic)
        with self.assertRaisesRegexp(StorageMethod.ResourceException,'^ticker nosymbol not found$'):
            method.acquireContent("{}/nosymbol".format(self.library_name),{})
        with self.assertRaisesRegexp(StorageMethod.ResourceException,'^library nolib not found in library list'):
            method.acquireContent("nolib/nosymbol".format(self.library_name),{})












from isharp.datahub.core import StorageMethod,AbstractDataBroker,Matrix,MatrixHeader,AcquireContentReturnValue
from nameko.testing.services import worker_factory
from isharp.datahub.broker_service.server import DataBrokerService
from unittest import TestCase
from unittest.mock import patch
from unittest.mock import MagicMock

test_header = MatrixHeader(name="hello", revision_id=5, storage_method="test", path=".", memory_style=None,
                           description="None")
test_content = {}

class ServerComponentTest(TestCase):

    @patch.multiple(StorageMethod, __abstractmethods__=set())
    def setUp(self):
        self.mock_storage_method = StorageMethod("test")
        self.broker = AbstractDataBroker(self.mock_storage_method)

    def tearDown(self):
        pass

    def test_service_checkout(self):
        testurl = "test:///file_name_1.csv?format=CSV"
        self.mock_storage_method.acquireContent=MagicMock(return_value=(AcquireContentReturnValue(header=test_header, content=test_content)))
        service = worker_factory(DataBrokerService, delegate=self.broker)
        result = service.checkout(testurl)
        self.assertEqual(5,result.matrix_header.revision_id)



from isharp.datahub.core import StorageMethod, MatrixHeader, Revision,MemStyles,AcquireContentReturnValue
import pandas as pd
import os
from typing import List
import logging

logger = logging.getLogger(__name__)

class FileStorageMethod(StorageMethod):
    allowed_formats = ["CSV"]
    def __init__(self, base_directory):
        self.base_directory = base_directory
        if os.path.isdir(self.base_directory):
            logging.info("Found base directory {}".format(self.base_directory))
            super().__init__("file")
        else:
            logger.error("could not read base directory {}".format(self.base_directory))
            raise StorageMethod.ResourceException()

    def storeContent(self, path, params, content,revision_info):
        super().storeContent(path, params, content,revision_info)
        file_path = os.path.join(self.base_directory, path.strip("/"))
        if (os.path.exists(file_path)):
            content.to_csv(file_path)
        else:
            raise StorageMethod.ResourceException("invalid file save destination {}".format(path))

    def make_header(self, path):
        csv_file_path = os.path.join(self.base_directory,path.strip("/"))
        index_path = os.path.join(os.path.dirname(csv_file_path),"index.txt")
        index_df = pd.read_csv(index_path, index_col=0)
        index_row = index_df.loc[path.strip("/"), :]
        return MatrixHeader(
                              name=index_row["name"],
                              revision_id=None,
                              storage_method=self.name,
                              memory_style=MemStyles.DATA_FRAME,
                              path = path[1:],
                              description=index_row['description'])

    def acquireContent(self, path, params,version_id=None):
        super().acquireContent(path,params)
        file_path = os.path.join(self.base_directory, path.strip("/"))
        logger.debug("Attempting to acquire file at [{}]  Base dir = [{}] original Path = [{}]".format(file_path,self.base_directory,path))
        if (os.path.exists(file_path)):
            content = pd.read_csv(file_path,index_col=0)
            last = content.index[-1]
            header = self.make_header(path)
            logger.info("Acquired file for {}".format(path))
            return AcquireContentReturnValue(content=content,header=header)
        else:
            logger.error("could not acquire file for {}".format(path))
            raise StorageMethod.ResourceException()


    def list(self):
        ret_val = []
        for dir_name,sub_dir_list,file_names in os.walk(self.base_directory):
            index_df =  pd.read_csv(os.path.join(dir_name,"index.txt"))
            for index,this_record in index_df.iterrows():
                header = MatrixHeader(
                name=this_record['name'],
                revision_id= None,
                storage_method= self.name,
                path=  this_record['path'] if 'path' in this_record else '',
                 # path='hello',
                memory_style=MemStyles.DATA_FRAME,
                description= this_record["description"]

                )
                ret_val.append(header)
        return ret_val
        # def __init__(self, name, revision_id, storage_method, url, memory_style):

    def history(self,matrix_url)->List[Revision]:
        return []


import unittest
import test.testutil.mongo_arctic_utils as marc
import test.testutil.log_utils as lu
import test.testutil.pandas_utils as pu
from isharp.datahub.arctic_broker.broker_impl.arctic_data_broker import ArcticBroker
from isharp.datahub.arctic_broker.broker_impl.arctic_storage_method import  add_revision_to_metadata,get_revisions_from_metadata,history_tag,import_pandas
from arctic import Arctic
import pandas as pd
import numpy as np
import datetime
from isharp.datahub.core import RevisionInfo

class TestArcticBroker(unittest.TestCase):

    def setUp(self):
        self.robot_user = "robot"
        self.import_comment = "importcomment"
        self.arctic = Arctic('localhost')
        self.library_name = marc.random_library(self.arctic)
        lu.logger.info("Created test library {}".format(self.library_name))
        self.arctic.initialize_library(self.library_name)
        simple_pd =pu.create_simple_series(['a','b','c'],5)
        lib = self.arctic[self.library_name]

        import_pandas(lib,simple_pd,"symbol",RevisionInfo(who=self.robot_user,what=self.import_comment, when = datetime.datetime.now()))
        import_pandas(lib,simple_pd,"ES.SETL.EOD",RevisionInfo(who=self.robot_user,what="import something else", when = datetime.datetime.now()))

    def tearDown(self):
        self.arctic.delete_library(self.library_name)
        lu.logger.info("deleted test library {}".format(self.library_name))


    def test_checkout_and_checkin_arctic(self):
        url = "arctic:///{}/symbol".format(self.library_name)
        broker = ArcticBroker(self.arctic)
        matrix = broker.checkout(url)
        history = broker.history(url)
        self.assertEqual(1, len(history))
        self.assertEqual(self.robot_user,history[0].revision_info.who)
        self.assertEqual("1", history[0].id)
        self.assertEqual(self.import_comment, history[0].revision_info.what)
        num_rows_original_version_1 = len(matrix.content.index)

        df  = matrix.content.append(pd.DataFrame(data=np.random.randn(1, len(matrix.content.columns)), index=[matrix.content.index[-1] + datetime.timedelta(days=1)],columns=matrix.content.columns))
        revision_info = RevisionInfo(who="Jeremy Ward", what="first test commit", when=datetime.datetime(year=2000,month=1,day=13))
        broker.commit(matrix.replace_content(df),revision_info)
        hist = broker.history(url)
        self.assertEqual(2,len(hist))
        self.assertEqual(hist[1].revision_info.what,"first test commit")
        self.assertEqual("2", hist[1].id)
        matrix = broker.checkout(url)


        num_rows_original_version_2 = len(matrix.content.index)
        self.assertEquals(1, num_rows_original_version_2 - num_rows_original_version_1)
        broker.release(matrix)
        original_version = broker.history(url)[0].id
        matrix = broker.checkout(url, original_version)
        self.assertEqual(num_rows_original_version_1,len(matrix.content.index))
        broker.release(matrix)

        version_2 = broker.history(url)[1].id
        matrix = broker.checkout(url, version=version_2)
        self.assertEquals(num_rows_original_version_2,len(matrix.content.index))
        broker.release(matrix)
        self.assertEqual(2,len(broker.history(url)))


    def test_list(self):
        broker = ArcticBroker(self.arctic)
        result = broker.list()
        self.assertEquals(2, len(result))
        self.assertEqual("{}/ES/SETL/EOD".format(self.library_name),result[0].path)


    def test_peek_with_existing_file(self):
        broker = ArcticBroker(self.arctic)

        url = "arctic:///{}/symbol".format(self.library_name)
        preview = broker.peek(url)

        todays_date = datetime.datetime.now().date()

        expected_start_date =  todays_date- datetime.timedelta(5)
        expected_end_date = expected_start_date + datetime.timedelta(4)

        self.assertEqual(expected_start_date.strftime("%Y-%m-%d"), preview.range_start.strftime("%Y-%m-%d"))
        self.assertEqual(expected_end_date.strftime("%Y-%m-%d"), preview.range_end.strftime("%Y-%m-%d"))

    def test_peek_non_existing_file(self):
        broker = ArcticBroker(self.arctic)
        testurl = "arctic:///subdir_1/file_name_xxx.csv?format=CSV"
        preview = broker.peek(testurl)
        self.assertIsNone(preview)

    def test_with_qualified_ticker_name(self):
        url = "arctic:///{}/ES/SETL/EOD".format(self.library_name)
        broker = ArcticBroker(self.arctic)
        matrix = broker.checkout(url)
        self.assertEqual(matrix.matrix_header.path,"/{}/ES/SETL/EOD".format(self.library_name))


    def tearDown(self):
        self.arctic.delete_library(self.library_name)
        lu.logger.info("deleted test library {}".format(self.library_name))








from unittest import TestCase

from isharp.datahub.core import MatrixUrl


class TestMatrixUrl(TestCase):
    def test_params(self):
        self.assertEqual(
            MatrixUrl("svn://myhost.co.uk:8080/root/index.csv").params(),
            {},
            "test a url without parameters")

        self.assertEqual(
            MatrixUrl("svn://myhost.co.uk:8080/root/inex.csv?param1=value1").params(),
            {"param1": "value1"},
            "testing a url with parameters")

        self.assertEqual(
            MatrixUrl("svn:///root/inex.csv?param1=value1").params(),
            {"param1": "value1"},
            "testing a url with parameters wihout")

        self.assertEqual(
            MatrixUrl("svn://?param1=value1").params(),
            {"param1": "value1"},
            "testing a url with parameters without host or path")

        self.assertEqual(
            MatrixUrl("svn://?param1=").params(),
            {},
            "testing a url with blank param")

        self.assertEqual(
            MatrixUrl("svn://?param1=value1&param2=value2").params(),
            {"param1": "value1","param2": "value2"},
            "testing a url with multi params")


    def test_host(self):
        self.assertEqual(
            MatrixUrl("svn://myhost/path/path2/path3?param1=value1&param2=value2").host(),
            "myhost",
            "testing host name")

        self.assertEqual(
            MatrixUrl("svn://myhost:8080/path/path2/path3").host(),
            "myhost",
            "testing host name with a port number")

        self.assertEqual(
            MatrixUrl("svn:///path/path2/path3").host(),
            None,
            "testing host name with blank host")


    def test_path(self):
        self.assertEqual(
            MatrixUrl("svn://myhost/path/path2/path3?param1=value1&param2=value2").path(),
            "/path/path2/path3",
            "testing path")

        self.assertEqual(
            MatrixUrl("htpp://localhost:8080").path(),
            "",
            "testing  empty path")

    def test_port(self):
        self.assertEqual(
            MatrixUrl("svn://myhost/path/path2/path3?param1=value1&param2=value2").port(),
            None,
            "testing empty port number")

        self.assertEqual(
            MatrixUrl("svn://myhost:8080/path/path2/path3?param1=value1&param2=value2").port(),
            8080,
            "testing non-empty port number")

    def test_scheme(self):
        self.assertEqual(
            MatrixUrl("svn://myhost/path/path2/path3?param1=value1&param2=value2").scheme(),
            "svn",
            "testing scheme")

        self.assertEqual(
            MatrixUrl("myhost:8080/path/path2/path3?param1=value1&param2=value2").port(),
            None,
            "testing empty scheme")



import luigi
import time
from multiprocessing import Process
from isharp.flow.core import CalculationTask
from isharp.flow.neo4jflow.py2neoflow import  calcTasks
from isharp.datahub.broker_client.remote_proxy import  BrokerConnectionPool



class DataHubTarget(luigi.Target):
    def __init__(self,data_hub_requirement,data_broker):
        self.data_hub_requirement = data_hub_requirement
        self.data_broker = data_broker

    def exists(self):

        peeked =  self.data_broker.peek(self.data_hub_requirement.url)
        print(" would be checking for url {} -> {}".format(self.data_hub_requirement.url,peeked))
        return peeked != None


class DataHubInputTask(luigi.ExternalTask):
    task_namespace = "isharp"
    task_family="DataRequirement"
    url = luigi.Parameter()
    t = luigi.IntParameter()

    def registerTarget(self,requirement,data_broker):
        self.data_hub_target = DataHubTarget(requirement,data_broker)

    def output(self):
        return self.data_hub_target



class LuigiCalculationTask(luigi.Task):
    accepts_messages = True
    task_namespace = "isharp"
    task_family="StrategyCalculation"
    strategy = luigi.Parameter()
    dueBy = luigi.Parameter()
    eval_label=luigi.Parameter()

    def set_requirements(self,requirements):
        self.requirements = requirements


    def run(self):
        progress = 0
        while True:
            progress += 1
            self.set_progress_percentage(progress)
            self.set_status_message("Doing stage {}".format(progress))
            time.sleep(1)
            print("_____________checking messages for job {} {} ".format(self.strategy,self.dueBy))
            if not self.scheduler_messages.empty():
                msg = self.scheduler_messages.get()
                print ("received {}".format(msg.content))
                if msg.content == "terminate":
                    break
                else:
                    msg.respond("unknown message")
        print ("finished")


    def requires(self):
        return self.requirements


def buildTask(calc_task,data_broker):
    task = LuigiCalculationTask(strategy=calc_task.strategy, dueBy=calc_task.dueBy, eval_label=calc_task.dueBy)
    requirements = []
    for reqiurement in calc_task.requirements:
          input_task = DataHubInputTask(url=reqiurement.url, t=reqiurement.t)
          input_task.registerTarget(reqiurement,data_broker)
          requirements.append(input_task)
    task.set_requirements(requirements)
    return task



def submit(task:CalculationTask):
    luigi.build([task])



if __name__ == '__main__':
    broker = BrokerConnectionPool()
    for calc_task in calcTasks("datahub","5672"):
        task = buildTask(calc_task,broker)
        p= Process(target=submit, args=(task,))
        p.start()












import random
import string
def randomString(stringLength=10):
    """Generate a random string of fixed length """
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for i in range(stringLength))



from isharp.datahub.broker_client.remote_proxy import BrokerConnectionPool

from flask import Flask, render_template
import os
import socket
import json
hostname=socket.gethostname()
from flask import request

templates_dir =  os.getenv('isharp_web_templates', 'templates')
print ("templates_dir: {}".format(templates_dir))
app = Flask(__name__,template_folder=templates_dir)

hub_host =  os.getenv('isharp_hub_host', 'localhost:5672')


@app.route('/')
def listing():
    brokers = [["Demo Broker","Prod parallel demo broker"],["UAT","UAT broker"],["DEV","Dev broker"]]
    listings = []
    with BrokerConnectionPool() as broker:
        for thisItem in broker.list(hub_host):
            listings.append(thisItem)
    return render_template('index.html',hostname="rabbit", my_list=listings,hub_host="Demo  Data Broker", brokers=brokers)


@app.route('/datahub/view/<path:path>', methods=['GET'])
def view(path):
    databroker = BrokerConnectionPool()
    protocol = request.args.get('protocol')
    url = "{}://{}/{}".format(protocol,hub_host,path)
    mtx = databroker.view(url)

    row_data = []
    dict_array = mtx.content.to_dict(orient='records')
    for idx, row in enumerate(dict_array):
        row["date"]=mtx.content.index[idx]
        row_data.append(row)

    column_headers = list(mtx.content)
    column_headers.insert(0,"date")
    history = databroker.history(url)


    return render_template('matrix.html',
                           column_headers=column_headers,
                           row_data = row_data,
                           revision_list = history)




from isharp.datahub.core import DataBroker,Matrix,AbstractDataBroker
from .arctic_storage_method import ArcticStorageMethod
class ArcticBroker(AbstractDataBroker):
    def __init__(self,store):
        super().__init__(ArcticStorageMethod(store))

    def checkout(self, url,version=None):
        return super().checkout(url,version)

    def commit(self, matrix, revisionInfo):
        super().commit(matrix, revisionInfo)










from enum import Enum
from typing import List
import dataclasses
from urllib.parse import urlparse
from urllib.parse import parse_qsl
import datetime
import abc
import luigi



import logging
logger = logging.getLogger(__name__)



from collections import namedtuple
class MemStyles(Enum):
    DATA_FRAME = 1
    TREE = 2


@dataclasses.dataclass(frozen=True)
class RevisionInfo:
    who: str
    what: str
    when: datetime.datetime





@dataclasses.dataclass(frozen=True)
class Revision:
    id: str
    revision_info: RevisionInfo












class MatrixUrl:
    def __init__(self,url):
        self.url = url
        self.url_components = urlparse(url)
        logger.debug("Parsed Url: host:[{}] path:[{}] port:[{}] scheme:[{}]".format(self.host(),self.path(),self.port(),self.scheme()))

    def params(self):
        params = parse_qsl(self.url_components.query)
        ret_val = {}
        for this_param in params:
            ret_val[this_param[0]] = this_param[1]
        return ret_val

    def host(self):
        return self.url_components.hostname

    def path(self):
        return  self.url_components.path

    def port(self):
        return self.url_components.port

    def scheme(self):
        return self.url_components.scheme


@dataclasses.dataclass(frozen=True)
class MatrixHeader:
    name: str
    revision_id: str
    storage_method: str
    path: str
    memory_style:MemStyles
    description: str

@dataclasses.dataclass(frozen=True)
class MatrixPreview:
    header: MatrixHeader
    range_start: str
    range_end: str


@dataclasses.dataclass(frozen=True)
class Matrix:
    matrix_header: MatrixHeader
    content: object
    url: MatrixUrl

    def replace_content(self,new_content):
        return Matrix(matrix_header=self.matrix_header,url=self.url,content=new_content)



class DataBroker(abc.ABC):

    class CheckoutException(Exception):
        pass

    class ProtocolException(Exception):
        pass

    @abc.abstractmethod
    def checkout(self, url:str, version_id=None)->Matrix:
        pass

    @abc.abstractmethod
    def view(self, url:str, version_id=None)->Matrix:
        pass

    @abc.abstractmethod
    def history(selfurl:str)->List[Revision]:
        pass


    @abc.abstractmethod
    def commit(self,matrix:Matrix,revisionInfo:RevisionInfo)->Revision:
        pass


    def release(self,matrix)->None:
        pass

    @abc.abstractmethod
    def list(self)->List[MatrixHeader]:
        pass


    @abc.abstractmethod
    def peek(self,url)->MatrixPreview:
        pass


    @abc.abstractmethod
    def releaseAll(self)->None:
        pass



AcquireContentReturnValue = namedtuple('AcquireContentReturnValue', 'header content')


class StorageMethod(abc.ABC):
    class ParameterException(Exception):
        pass
    class ResourceException(Exception):
        pass

    def __init__(self, name):
        self.name = name

    @abc.abstractmethod
    def acquireContent(self,path,params,version_id=None)->AcquireContentReturnValue:
        pass


    @abc.abstractmethod
    def storeContent(self,path,params,content,revision_info)->Revision:
        pass


    @abc.abstractmethod
    def list(self)->List[MatrixHeader]:
        pass


class AbstractDataBroker(DataBroker):
    def __init__(self,storage_method):
        super().__init__()
        self.storage_method = storage_method
        self.register = []

    def _assert_not_checked_out(self,matrix_url):
        if matrix_url.path() in self.register:
            raise DataBroker.CheckoutException("matrix [{}] is already checked out".format(matrix_url.path()))

    def _assert_checked_out(self,matrix_url):
        if matrix_url.path() not in self.register:
            raise DataBroker.CheckoutException("matrix [{}] is not already checked out".format(matrix_url.path()))

    def _check_schema(self,matrix_url):
        if matrix_url.scheme() != self.storage_method.name:
            raise DataBroker.ProtocolException()

    def _open(self, matrix_url_str,checkout,version=None ):
        logger.debug("Abstract  broker called with {} checkout ? {}".format(matrix_url_str,checkout))
        matrix_url = MatrixUrl(matrix_url_str)
        self._check_schema(matrix_url)
        if checkout:
            self._assert_not_checked_out(matrix_url)
            self.register.append(matrix_url.path())
        result = self.storage_method.acquireContent(path=matrix_url.path(), params=matrix_url.params(),version_id=version)
        ret_val =Matrix(result.header,result.content,matrix_url)
        logger.debug("Abstract data broker about to return matrix")
        return ret_val

    def checkout(self, matrix_url_str,version=None):
        return self._open(matrix_url_str,True,version)


    def history(self,url: str) -> List[Revision]:
        matrix_url = MatrixUrl(url)
        self._check_schema(matrix_url)
        return self.storage_method.history(matrix_url)

    def view(self, matrix_url_str,version=None):
        return self._open(matrix_url_str, False,version)

    def commit(self, matrix, revisionInfo):
        self._assert_checked_out(matrix.url)
        revision = self.storage_method.storeContent(path=matrix.matrix_header.path,content=matrix.content,revision_info=revisionInfo,params=matrix.url.params())
        self.register.remove(matrix.url.path())
        return revision

    def releaseAll(self):
        logger.debug("Abstract databroker about to release all")
        self.register.clear()

    def release(self, matrix):
        self.register.remove(matrix.url.path())

    def list(self):
        return self.storage_method.list()

    def peek(self, url) -> MatrixPreview:
        matrix_url = MatrixUrl(url)
        self._check_schema(matrix_url)
        try:
            content_tuple = self.storage_method.acquireContent(path=matrix_url.path(), params=matrix_url.params(),version_id=None)
            return  MatrixPreview(header =  content_tuple.header, range_start=content_tuple.content.index[0], range_end=content_tuple.content.index[-1])
        except StorageMethod.ResourceException:
            return None


class CombiBroker(DataBroker):
    def __init__(self,registry):
        self.registry = registry

    def checkout(self, url, version_id=None) -> Matrix:
        logger.info("combi broker called with {}".format(url))
        parsed_url = MatrixUrl(url)
        return self._delegate(parsed_url.scheme()).checkout(url,version_id)

    def history(self ,url:str ) -> List:
        logger.info("combi broker history called with {}",url)
        parsed_url = MatrixUrl(url)
        return self._delegate(parsed_url.scheme()).history(url)

    def view(self, url, version_id=None) -> Matrix:
        logger.info("combi broker View called with {}".format(url))
        parsed_url = MatrixUrl(url)
        return self._delegate(parsed_url.scheme()).view(url,version_id)


    def peek(self, url) -> MatrixHeader:
        logger.info("combi broker called with peek {}".format(url))
        parsed_url = MatrixUrl(url)
        return self._delegate(parsed_url.scheme()).peek(url)

    def _delegate(self,scheme)->DataBroker:
        if (scheme in self.registry):
            return self.registry[scheme]
        else:
            raise DataBroker.ProtocolException("invalid protocol: {}".format(scheme))


    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        logger.info("combi broker  commit called ")
        return self._delegate(matrix.url.scheme()).commit(matrix,revisionInfo)

    def release(self, matrix) -> None:
        logger.info("combi broker  release called")
        self._delegate(matrix.url.scheme()).release(matrix)
        return None

    def releaseAll(self) -> None:
        logger.info("combi broker  release called")
        for broker in self.registry.values():
            broker.releaseAll()

    def list(self) -> List[MatrixHeader]:
        logger.info("combi broker list method called")
        ret_val = []
        for this_delegate in self.registry.values():
            logger.info("getting listing from delegate")
            this_pack = this_delegate.list()
            for listing_item in this_pack:
                ret_val.append(listing_item)

        logger.info("combi broker returning listing with {} values".format(len(ret_val)))
        return ret_val



@dataclasses.dataclass(frozen=True)
class DatahubTarget():
    url: str
    t: int


    def exists(self,data_broker)->bool:
        return data_broker.peek(self.url) != None




from isharp.datahub.broker_client.remote_proxy import BrokerConnectionPool
from  isharp.datahub.broker_client.client_utils import mtx_headers_as_dataframe as to_df
import logging
import json
logging.basicConfig()
logger = logging.getLogger()


# create console handler and set level to info
handler = logging.StreamHandler()
handler.setLevel(logging.INFO)
formatter = logging.Formatter("%(levelname)s - %(message)s")
handler.setFormatter(formatter)
logger.addHandler(handler)

logger = logging.getLogger(__name__)


logger.info("hello from logging")

aws_rpc_host = 'isharpdev'
local_host = "localhost"
rpc_host = '3.86.185.181'
with BrokerConnectionPool() as broker:
    # broker.releaseAll()
    # mtx = broker.checkout("file://datahub:5672/file_name_1.csv?format=CSV")
    # print(mtx.content)
    # broker.release(mtx)
    # mtx = broker.checkout("file://datahub:5672/file_name_1.csv?format=CSV")
    # broker.release(mtx)
    # # print (to_df(broker.list("datahub:5672")))
    # # print(broker.peek("file://datahub:5672/file_name_1.csv?format=CSV"))
    for thisItem in broker.list('{}:5672'.format(rpc_host)):
        print (thisItem)

    broker.releaseAll()
    mtx = broker.checkout('arctic://{}:5672/YahooFinance/SPOT/FTSE/0500'.format(rpc_host))
    print (mtx.matrix_header.path)

    broker.releaseAll()

    hist = broker.history('arctic://{}:5672/YahooFinance/SPOT/FTSE/0500'.format(rpc_host))

    hist = broker.history('arctic://{}:5672/isharp/AAPL'.format(rpc_host))
    # mtx = broker.view('arctic://{}:5672/InvestCo/CLOSING/SP/EOD'.format(rpc_host))





    # row_data = []
    # dict_array = mtx.content.to_dict(orient='records')
    # for idx, row in enumerate(dict_array):
    #     row["date"]=mtx.content.index[idx].strftime("%d %b %Y")
    #     row_data.append(row)
    #
    # column_headers = list(mtx.content)
    # column_headers.append("date")





    print ("finidhse")

















import py2neo
import os
import yaml
from py2neo import  Graph, Node, Relationship

path = os.path.dirname(os.path.realpath(__file__))
yaml_path =  os.path.join(path, "strategies.yaml")
with open(yaml_path, 'r') as stream:
    yam = yaml.safe_load(stream)


url = "bolt://ec2-34-205-159-121.compute-1.amazonaws.com:7687"
graph = Graph(url)
tx = graph.begin()
graph.delete_all()
instrument_nodes = {}
capture_nodes = {}

def new_node(label,**properties):
    newNode = Node(label,**properties)
    graph.create(newNode)
    return newNode
    graph.create(Relationship(from_node,name,to_node))


for market in yam['markets']:
    newMarket = new_node("Market", **market['props'])
    for instr in market['instrs']:
        new_instr = new_node ('Instrument',**instr)
        instrument_nodes[instr['name']]= new_instr
        graph.create(Relationship(new_instr ,"TRADES_ON",newMarket))


for (feedkey,feedVal) in yam['feedpoints'].items():
    newFeed = Node("PriceFeed", name=feedkey)
    graph.create(newFeed)
    for (fieldKey,fieldVal) in feedVal.items():
        newField = new_node("FeedField",name=fieldKey)
        feed_to_field = Relationship(newFeed,"HAS_FIELD",newField)
        graph.create(feed_to_field)
        for ticker in fieldVal['tickers']:
            newTicker = new_node("FeedTicker",name=ticker)
            field_to_ticker = Relationship(newField,"HAS_TICKER",newTicker)
            graph.create(field_to_ticker)
            for capture in fieldVal['times']:
                feed_path = '/'.join([feedkey,fieldKey,ticker,capture])
                capture_nodes[feed_path]=new_node("Capture", name=capture,path=feed_path)
                graph.create(Relationship(newTicker,"CAPTURES",capture_nodes[feed_path]))


for strat in yam['strategies']:
    newStrat = new_node("Strategy",**strat['props'])
    for quote in strat['quotes']:
        quote_to_strat = Relationship(new_node('Quote',**quote['props']),"PUBLISHED_BY",newStrat)
        graph.create(quote_to_strat)
    for constituent in strat['constituents']:
        graph.create(Relationship(instrument_nodes[constituent['name']],"CONSTITUENT_OF", newStrat,**constituent['props']))

    for eval in strat['evals']:
        new_job = new_node("Job",**eval['props'])
        graph.create(Relationship(newStrat,"EVAL_JOB",new_job))
        for (capture_key,capture_path) in eval['dependencies'].items():
            amended_path = capture_path[0:2]
            amended_path.append(capture_path[3])
            amended_path.append(capture_path[2])
            path = '/'.join(amended_path)
            capture_node = capture_nodes[path]
            graph.create(Relationship(new_job,"USES",capture_node))
            graph.create(Relationship(capture_node,"OBSERVATION_OF",instrument_nodes[capture_key]))



tx.commit()


import dataclasses
import isharp.datahub.broker_client
from  isharp.datahub.core import DataBroker
from typing import List
@dataclasses.dataclass(frozen=True)
class DatahubRequirement:
    name:str
    url:str
    t: int


@dataclasses.dataclass(frozen=True)
class CalculationTask:
    requirements: List[DatahubRequirement]
    strategy: str
    dueBy: str
    eval_label: str










import luigi
import time
import datetime
from multiprocessing import Process
from isharp.flow.core import CalculationTask
from isharp.flow.neo4jflow.py2neoflow import  calcTasks
from typing import List

from luigi import rpc


def getRemoteScheduler():
    return rpc.RemoteScheduler("http://localhost:8082")



if __name__ == '__main__':
    sch = getRemoteScheduler()
    workers = sch.worker_list()
    tasks = sch.task_list()
    gr = sch.graph()
    print (gr)
    print(workers)
    for (taskKey,taskValue) in tasks.items():
        print(taskValue)


















import pandas as pd
def header_cols(header):
    return (header.name,
            header.storage_method,
            header.path,
            str(header.memory_style),
            header.description)

def mtx_headers_as_dataframe(matrix_headers):
    record_data = [header_cols(i) for i in matrix_headers]
    return pd.DataFrame.from_records(data= record_data, columns= ["name", "storage method", "path", "mem style", "description"])



#!/usr/local/bin/python
# -*- coding: utf-8 -*-
import re
import sys
import isharp.datahub.yaml_support  as iYaml
import nameko.cli.main
import isharp.datahub.web.webconsole as web
from multiprocessing import Process

def main():
    web.app.run(host='0.0.0.0',port=80)

if __name__== "__main__" :
    main()





from distutils.core import setup
from setuptools import find_packages
setup(
    name='isharp',
    packages=find_packages(),
    version='2.68',
    description='Algorithmic workbench',
    author='jeremycward',
    author_email='jeremycward@yahoo.co.uk',
    url='https://github.com/jeremycward/isharp-core',
    download_url='https://github.com/jeremycward/isharp-core/tarball/1.00',
    keywords=[],
    classifiers=[],
    entry_points={
        'console_scripts':[
            'datahub=isharp.datahub.broker_service.datahub_main:main',
            'web=isharp.datahub.broker_service.datahub_webserver_main:main'
        ]
    },
)



import os
import shutil
import tempfile
import unittest
import pandas as pd
import datetime
import test.testutil.file_utils as fu

import test.testutil.log_utils as lu
from isharp.datahub.core import StorageMethod, MemStyles
from isharp.datahub.csv_files.simple_file_broker import SimpleFileBroker

testpath = os.path.dirname(__file__)

class TestFileDataBroker(unittest.TestCase):

    def setUp(self):
        rows = ['a','b','c','d','e','f','g','h','i','j','k']
        self.test_data_path = tempfile.mkdtemp(prefix="ipp_test_file_data_broker")
        lu.logger.info("test file path: {}" + self.test_data_path)
        fu.make_file_tree(self.test_data_path,2,3)
        self.broker = SimpleFileBroker(self.test_data_path)


    def tearDown(self):
        shutil.rmtree(self.test_data_path)



    def test_invalid_path(self):
        with self.assertRaises(StorageMethod.ResourceException):
            self.broker.checkout("file://broker.nomura.com/no_dir/no_file?format=CSV")

    def test_get_compound_path(self):
        testurl = "file:///subdir_1/file_name_1.csv?format=CSV"
        m = self.broker.checkout(testurl)
        self.assertEqual("file_name_1.csv",m.matrix_header.name)
        self.assertEqual("subdir_1/file_name_1.csv", m.matrix_header.path)



    def test_peek_with_existing_file(self):
        testurl = "file:///subdir_1/file_name_1.csv?format=CSV"
        preview = self.broker.peek(testurl)

        todays_date = datetime.datetime.now().date()

        expected_start_date =  todays_date- datetime.timedelta(11)
        expected_end_date = expected_start_date + datetime.timedelta(10)

        self.assertEqual(expected_start_date.strftime("%Y-%m-%d"), preview.range_start)
        self.assertEqual(expected_end_date.strftime("%Y-%m-%d"), preview.range_end)


    def test_peek_non_existing_file(self):
        testurl = "file:///subdir_1/file_name_xxx.csv?format=CSV"
        preview = self.broker.peek(testurl)
        self.assertIsNone(preview)


    def test_get_simple_matrix(self):
        testurl = "file:///file_name_1.csv?format=CSV"
        m = self.broker.checkout(testurl)
        self.assertEqual("file_name_1.csv",m.matrix_header.name)
        self.assertEqual(None,m.matrix_header.revision_id)
        self.assertEqual('file', m.matrix_header.storage_method)
        self.assertEqual(m.matrix_header.path, "file_name_1.csv")
        self.assertTrue(isinstance(m.content,pd.DataFrame))
        self.assertEqual(MemStyles.DATA_FRAME, m.matrix_header.memory_style)

    def test_list(self):
        headers = self.broker.list()
        self.assertEquals(14,len(headers))
        header  = headers[0]
        self.assertIsNone(header.revision_id)
        self.assertEqual("file",header.storage_method)
        self.assertEqual("file_name_1.csv", header.path)
        self.assertEqual("file_name_1.csv",header.name)
        self.assertEqual("description of file_name_1.csv",header.description)
        self.assertEqual(MemStyles.DATA_FRAME, header.memory_style)




import abc
import luigi
import time
from nameko.rpc import rpc, RpcProxy
from typing import List
import logging
from isharp.datahub.core import DatahubTarget
logger = logging.getLogger(__name__)



class EvalMethod(object):
    pass

class Evaluator(abc.ABC):
    @abc.abstractmethod
    def eval(self, method:EvalMethod ,inputs: List[DatahubTarget])->List[DatahubTarget]:
        pass



class EvalService(Evaluator):
    name="evaluation_service"

    @rpc
    def eval(self, method: EvalMethod, inputs: List[DatahubTarget]) -> List[DatahubTarget]:
        logger.info("START performing an eval....")
        time.sleep(1)
        logger.info("END performing an eval....")
        return [DatahubTarget(url="url",t=0)]







from neo4j import GraphDatabase
import luigi
import os
from isharp.flow.core import DatahubRequirement
from isharp.flow.core import CalculationTask


class JobFetcher(object):

    def __init__(self, uri, user, password):
        self._driver = GraphDatabase.driver(uri, auth=(user, password))

    def close(self):
        self._driver.close()

    def fetchJobs(self):
        with self._driver.session() as session:
            queryStr = """"
            match (s:Strategy)-->(p:Job)-->(c:Capture)<--(ticker:FeedTicker)<--(field:FeedField)<--(feed:PriceFeed) return 
            
            p,s,c,ticker,field,feed
            
            """

            records = session.write_transaction(lambda tx:tx.run(queryStr))
            for record in records:
                print(record)

    def reducedJobs(self):
        with self._driver.session() as session:
            queryStr = "match  p= (s:Strategy)-->(j:Job)-->(c:Capture)<--(ticker:FeedTicker)<--(field:FeedField)<--(feed:PriceFeed)  return reduce (stratMap = 0, n IN nodes(p) | stratMap + 1  )  AS reduction"

            records = session.write_transaction(lambda tx: tx.run(queryStr))
            for record in records:
                print(record)


jf = JobFetcher("bolt://ec2-34-205-159-121.compute-1.amazonaws.com:7687",user="",password="")
jf.reducedJobs()
jf.close()



import shutil
import os
import subprocess
import tempfile
import pymongo
import time
from pymongo import MongoClient
from arctic import Arctic

class MongoTemporaryInstance(object):

    def __init__(self,MONGODB_TEST_PORT):
        self._tmpdir = tempfile.mkdtemp()
        self._process = subprocess.Popen(['mongod', '--bind_ip', 'localhost',
                                          '--port', str(MONGODB_TEST_PORT),
                                          '--dbpath', self._tmpdir,
                                          '--nojournal', '--nohttpinterface',
                                          '--noauth', '--smallfiles',
                                          '--syncdelay', '0',
                                          '--maxConns', '10',
                                          '--nssize', '1', ],
                                         stdout=open(os.devnull, 'wb'),
                                         stderr=subprocess.STDOUT)

        # XXX: wait for the instance to be ready
        #      Mongo is ready in a glance, we just wait to be able to open a
        #      Connection.
        for i in range(3):
            time.sleep(0.1)
            try:
                client = MongoClient('localhost',MONGODB_TEST_PORT)
                print(client.server_info())
                # self.arctic = Arctic(MongoClient('localhost',MONGODB_TEST_PORT))

            except pymongo.errors.ConnectionFailure:
                continue
            else:
                break
        else:
            assert False, 'Cannot connect to the mongodb test instance'


    def shutdown(self):
        if self._process:
            self._process.terminate()
            self._process.wait()
            self._process = None
            shutil.rmtree(self._tmpdir, ignore_errors=True)





from typing import List
from isharp.datahub.core import DataBroker, MatrixHeader, Matrix, AcquireContentReturnValue, Revision
from isharp.datahub.core import MatrixPreview
from isharp.datahub.core import RevisionInfo
from urllib.parse import urlparse
from nameko.standalone.rpc import ClusterRpcProxy
import logging

logger = logging.getLogger(__name__)




def remote_config(net_location: str):
    return {
        'serializer': 'pickle',
        'AMQP_URI': 'pyamqp://guest:guest@{}'.format(net_location),
        'rpc_exchange': 'nameko-rpc',
        'max_workers': 10,
        'parent_calls_tracked': 10
    }


class PooledBrokerConnection(DataBroker):
    def __init__(self, net_location: str):
        logger.info("creating remote client at {}".format(net_location))
        conf = remote_config(net_location)
        self.rpc_proxy = ClusterRpcProxy(conf)
        self.proxy = self.rpc_proxy.start()
        self.net_location = net_location

    def releaseAll(self) -> None:
        logger.info("release all for  remote client at {}".format(self.net_location))
        self.proxy.data_broker_service.releaseAll()

    def stop(self):
        logger.info("closing remote client at {}".format(self.net_location))
        self.rpc_proxy.stop()

    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        return self.proxy.data_broker_service.commit(matrix, revisionInfo)

    def history(self,url: str) -> List[Revision]:
        return self.proxy.data_broker_service.history(url)

    def list(self) -> List[MatrixHeader]:
        return self.proxy.data_broker_service.list()

    def checkout(self, url: str, version_id=None) -> Matrix:
        return self.proxy.data_broker_service.checkout(url)

    def view(self, url: str, version_id=None) -> Matrix:
        return self.proxy.data_broker_service.view(url)

    def release(self, matrix) -> None:
        self.proxy.data_broker_service.release(matrix)
        return None

    def peek(self, url) -> MatrixPreview:
        return self.proxy.data_broker_service.peek(url)


class BrokerConnectionPool(DataBroker):
    def __init__(self):
        self.pool = {}

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        for thisConnection in self.pool.values():
            thisConnection.stop()

    def _acquire_connection(self, connection_key):
        if connection_key not in self.pool.keys():
            self.pool[connection_key] = PooledBrokerConnection(connection_key)
        return self.pool[connection_key]

    def _connect(self, url: str):
        connection_key = self._conn_details(url)
        return self._acquire_connection(connection_key)

    def _conn_details(self, url: str) -> str:
        url_components = urlparse(url)
        return url_components.netloc

    def checkout(self, url: str, version_id=None) -> Matrix:
        return self._connect(url).checkout(url, version_id)

    def history(self,url: str) -> List[Revision]:
        return self._connect(url).history(url)


    def view(self, url: str, version_id=None) -> Matrix:
        return self._connect(url).view(url, version_id)

    def commit(self, matrix: Matrix, revisionInfo: RevisionInfo) -> Revision:
        return self._connect(matrix.url.url).commit(matrix, revisionInfo)

    def release(self, matrix) -> None:
        self._connect(matrix.url.url).release(matrix)

    def releaseAll(self) -> None:
        for thisConnection in self.pool.values():
            thisConnection.releaseAll()

    def peek(self, url) -> MatrixPreview:
        return self._connect(url).peek(url)

    def list(self, network_location):
        return self._acquire_connection(network_location).list()






import dataclasses
from typing import Dict
from typing import List

from typing import Type
from typing import Any




@dataclasses.dataclass(frozen=True)
class ParameterFieldDefn:
    name:str
    caption:str
    description:str
    param_type: Type
    required: bool
    default: Any

@dataclasses.dataclass(frozen=True)
class ParameterField:
    defn: ParameterFieldDefn
    value: Any


@dataclasses.dataclass(frozen=True)
class ParameterSet:
    name: str
    description:str
    content:List[ParameterField]





@dataclasses.dataclass(frozen=True)
class IndexConsituent:
    name:str


@dataclasses.dataclass(frozen=True)
class AlgoIndex:
    name:str
    constituentMeta: List[ParameterFieldDefn]
    indexMeta: List[ParameterFieldDefn]
    constituents:List[IndexConsituent]







from neo4j import GraphDatabase
import luigi
import os


feedpoints = {
    "InvestCo": {
        "CLOSING": {"times": ["EOD"],
                    "tickers": ["NIK", "RUSS", "FTSE", "NDX", "EURO","SP"]
                    },
        "SPOT": {"times": ["0500", "0900", "1000", "1730"],
                 "tickers": ["NIK", "RUSS", "FTSE", "NDX", "EURO","SP"]

                 },

    },

    "YahooFinance": {
        "CLOSING": {"times": ["EOD"],
                    "tickers": ["NIK", "RUSS", "FTSE", "NDX", "EURO","SP"]
                    },
        "SPOT": {"times": ["0500", "0900", "1000", "1730"],
                 "tickers": ["NIK", "RUSS", "FTSE", "NDX", "EURO","SP"]

                 },
    },

}

requirements = {
    'minX': {
        '2130': {
            'Nikkei': ['InvestCo', 'CLOSING', 'EOD', 'NIK'],
            'NDX': ['InvestCo', 'CLOSING', 'EOD', 'NDX'],
            'SanP500': ['InvestCo', 'CLOSING', 'EOD', 'SP']
        },

        '0500': {
            'Nikkei': ['YahooFinance', 'SPOT', '0500', 'NIK'],
            'NDX': ['InvestCo', 'CLOSING', 'EOD', 'NDX'],
            'SanP500': ['InvestCo', 'CLOSING', 'EOD', 'SP']
        }
    }
}


def inject_requirements_tx(tx, requirements):
    for (strat_name,requirements_map) in requirements.items():
        for (job_name, job_requirements) in requirements_map.items():
            for (instr_name, requirement_path) in job_requirements.items():
                find_instr_phrase = "MATCH (n:Instrument {{name:'{}'}}) return n".format(instr_name)
                instr_node_id =  tx.run(find_instr_phrase).single()['n'].id
                find_job_phrase = "MATCH (n:Job {{due_by:'{}'}}) return n".format(job_name)
                job_node_id = tx.run(find_job_phrase).single()['n'].id
                find_feed_point_prhase = "MATCH (PriceFeed {{name: '{}'}})--(FeedField {{ name: '{}'}})--(FeedTicker {{name: '{}'}})--(n: Capture {{name:'{}'}}) return n"\
                    .format(requirement_path[0],requirement_path[1],requirement_path[3],requirement_path[2])
                find_results =tx.run(find_feed_point_prhase)
                feed_point_id = find_results.single()['n'].id

                observation_of_link_prhase = "MATCH (instr),(feedpoint) where id(instr) = {} and id(feedpoint) = {} CREATE (feedpoint)-[r:OBSERVATION_OF]->(instr)".format(instr_node_id,feed_point_id)
                tx.run(observation_of_link_prhase)

                uses_link_phrase = "MATCH (job),(feedpoint) where id(job) = {} and id(feedpoint) = {} CREATE (job)-[r:USES]->(feedpoint)".format(job_node_id,feed_point_id)
                tx.run(uses_link_phrase)


def inject_feeds_tx(tx, feed_points):
    for (feed_name, feed_info) in feed_points.items():
        price_feed_phrase = "CREATE (z:PriceFeed {{name:'{}'}}) return z".format(feed_name, feed_name)
        feed_id_result = tx.run(price_feed_phrase)
        feed_id = feed_id_result.single()['z'].id
        for (field_value, field_value_info) in feed_info.items():
            field_phrase = "match(n) WHERE id(n) = {} CREATE(n)-[:HAS_FIELD]->(z:FeedField {{name:'{}'}}) RETURN z".format(
                feed_id, field_value, field_value)
            field_insert_result =  tx.run(field_phrase)
            field_id = field_insert_result.single()['z'].id
            for ticker in field_value_info['tickers']:
                add_ticker_phrase = "match(n) WHERE id(n) = {} CREATE(n)-[:HAS_TICKER]->(z:FeedTicker {{name:'{}'}} ) RETURN z".format(
                    field_id, ticker, ticker)
                result =  tx.run(add_ticker_phrase)
                ticker_id = result.single()['z'].id
                for time in field_value_info['times']:
                    time_insert_phrase = "match(n) WHERE id(n) = {} CREATE(n)-[:CAPUTURES]->(z:Capture {{name:'{}'}} ) RETURN z".format(
                        ticker_id, time, time)
                    tx.run(time_insert_phrase)


class DbSetup():
    def __init__(self, uri, user, password):
        self._driver = GraphDatabase.driver(uri, auth=(user, password))

    def close(self):
        self._driver.close()

    def clean(self):
        with self._driver.session() as session:
            return session.write_transaction(self.delete_all)

    def setUp(self):
        with self._driver.session() as session:
            return session.write_transaction(self.set_up)

    def inject_requirements (self,job_requirements):
        with self._driver.session() as session:
            return session.write_transaction(inject_requirements_tx, job_requirements)



    def insert_feeds(self, feed_points):
        with self._driver.session() as session:
            return session.write_transaction(inject_feeds_tx, feed_points)

    @staticmethod
    def set_up(tx):
        return tx.run(setup_query)

    @staticmethod
    def delete_all(tx):
        tx.run("match (n) detach delete n")


path = os.path.dirname(os.path.realpath(__file__))
cypher_path = os.path.join(path, "neo4jsetup.cypher")
with open(cypher_path, 'r') as cypher_file:
    setup_query = cypher_file.read()


yaml_path =  os.path.join(path, "neo4jsetup.cypher")

setup = DbSetup("bolt://isharpdemo:7687", user="", password="")

setup.clean()
setup.setUp()
setup.insert_feeds(feedpoints)
setup.inject_requirements(requirements)
setup.close()
